function scryptonite_boot()
{
	scryptonite_context = new render_context();
	scryptonite_context.fullscreen();
	
	var items = [
		new element_audio('audioplayer',true),
		new element_timed(7.5,15, new element_shader_with_texture("2d-vertex-shader-dummy", "2d-fragment-shader-intro", "img/powerteam.png")),
		new element_timed(15,30,new element_shader_with_texture('2d-vertex-shader-dummy','2d-fragment-shader-tunnel','img/maske.jpg')),
		new element_timed(30,45,new element_shader_with_texture('2d-vertex-shader-dummy','2d-fragment-shader-grid','img/talkinghand.jpg')),
		new element_timed(45,60,new element_shader_with_texture('2d-vertex-shader-dummy','2d-fragment-shader-spotlight','img/flag.png')),
		new element_timed(60,75,new element_shader_with_texture('2d-vertex-shader-dummy','2d-fragment-shader-wobbletunnel','img/fly.png')),
		new element_timed(75,98,new element_shader_with_texture('2d-vertex-shader-dummy','2d-fragment-shader-lava','img/bommelum.jpg')),
	];
	
	scryptonite_context.add_list(items);
	scryptonite_context.go();
}